@@ -1,29 +1,24 @@
-import os
 import math
 
 import numpy as np
 
 from typing import Any
+from typing import List
 from typing import Tuple
 from typing import Union
 from typing import Callable
 from typing import Optional
 from typing import NamedTuple
 from collections import Counter
+from multiprocessing.shared_memory import SharedMemory
 from numpy.lib.stride_tricks import as_strided
 
-from .misc import hash_code
-from .misc import print_warning
+from .misc import random_hash
 from .types import torch
 from .types import torchvision
 from .types import F
 from .types import tensor_dict_type
 
-try:
-    import SharedArray as sa
-except:
-    sa = None
-
 
 arr_type = Union[np.ndarray, torch.Tensor]
 
@@ -518,108 +513,27 @@ class StrideArray:
         return self._construct(repeated_shapes, repeated_strides)
 
 
-def _to_address(path: str) -> str:
-    return f"shared_{hash_code(path)}"
-
-
-def _check_sa(address: str) -> bool:
-    if sa is None:
-        return False
-    try:
-        sa.attach(address)
-        return True
-    except FileNotFoundError:
-        return False
-
-
-def _write_shared(address: str, array: np.ndarray) -> None:
-    try:
-        shared = sa.attach(address)
-    except FileNotFoundError:
-        shared = sa.create(address, array.shape, array.dtype)
-    shared[:] = array
-
-
-class SharedArrayWrapper:
-    def __init__(self, root: str, path: str, *, to_memory: bool):
-        self.path = os.path.join(root, path)
-        self.folder, file = os.path.split(self.path)
-        os.makedirs(self.folder, exist_ok=True)
-        self.flag_path = os.path.join(self.folder, f"flag_of_{file}")
-        self.address, self.flag_address = map(_to_address, [self.path, self.flag_path])
-        if to_memory and sa is None:
-            print_warning(
-                "`to_memory` is set to True "
-                "but `SharedArray` lib is not available, therefore "
-                "`to_memory` will be set to False"
-            )
-            to_memory = False
-        self.to_memory = to_memory
-
-    @property
-    def is_ready(self) -> bool:
-        if self.to_memory:
-            if not _check_sa(self.address):
-                return False
-            if not _check_sa(self.flag_address):
-                return False
-            return sa.attach(self.flag_address).item()
-        if not os.path.isfile(self.path):
-            return False
-        if not os.path.isfile(self.flag_path):
-            return False
-        return bool(np.load(self.flag_path, mmap_mode="r").item())
-
-    def read(self, *, writable: bool = False) -> np.ndarray:
-        if self.to_memory:
-            arr = sa.attach(self.address)
-            arr.flags.writeable = writable
-            return arr
-        return np.load(self.path, mmap_mode="r+" if writable else "r")
-
-    def write(self, arr: np.ndarray) -> None:
-        self._write(arr, overwrite=True, is_finished=True)
-
-    def prepare(self, arr: np.ndarray) -> None:
-        # prepare an empty array at certain path / address
-        # this is mainly for multiprocessing
-        self._write(arr, overwrite=False, is_finished=False)
-
-    def mark_finished(self) -> None:
-        flag = self.read(writable=True)
-        flag[0] = True
-
-    def delete(self) -> None:
-        if self.is_ready:
-            print(f"> removing {self.path} & {self.flag_path}")
-            if self.to_memory:
-                sa.delete(self.address)
-                sa.delete(self.flag_address)
-                return None
-            os.remove(self.path)
-            os.remove(self.flag_path)
-
-    def _give_permission(self) -> None:
-        os.system(f"chmod -R 777 {self.folder}")
-
-    def _write(
+class SharedArray:
+    def __init__(
         self,
-        arr: np.ndarray,
-        *,
-        is_finished: bool,
-        overwrite: bool = True,
-    ) -> None:
-        if self.is_ready and overwrite:
-            path = self.address if self.to_memory else self.path
-            print(f"> there's already an array at '{path}', " "it will be overwritten")
-            self.delete()
-        if self.to_memory:
-            _write_shared(self.address, arr)
-            _write_shared(self.flag_address, np.array([is_finished]))
-            return None
-        np.save(self.path, arr)
-        np.save(self.flag_path, np.array([is_finished]))
-        self._give_permission()
+        dtype: Union[type, np.dtype],
+        shape: Union[List[int], Tuple[int, ...]],
+        data: Optional[np.ndarray] = None,
+    ):
+        name = random_hash()
+        d_size = np.dtype(dtype).itemsize * np.prod(shape)
+        self._shm = SharedMemory(create=True, size=d_size, name=name)
+        self.value = np.ndarray(shape=shape, dtype=dtype, buffer=self._shm.buf)
+        if data is not None:
+            self.value[:] = data[:]
+
+    def destroy(self) -> None:
+        self._shm.close()
+        self._shm.unlink()
+
+    @classmethod
+    def from_data(cls, data: np.ndarray) -> "SharedArray":
+        return cls(data.dtype, data.shape, data)
 
 
 def get_label_predictions(logits: np.ndarray, threshold: float) -> np.ndarray:
